#ifndef __BOARD_H_
#define __BOARD_H_

#include "defs.h"

/* flags for castling.
   each side has two flags, one for its queen side rook
   and one for its king side rook. Moving the king resets
   both castle flags.
*/
#define FL_WQCASTLE (1<<0)
#define FL_WKCASTLE (1<<1)
#define FL_BQCASTLE (1<<2)
#define FL_BKCASTLE (1<<3)



/*
6 most significant bits of the char: Rank of piece
least 2 significant bits: color
*/


#define PAWN  1
#define ROOK  2
#define KNIGHT  3
#define BISHOP  4
#define QUEEN  5
#define KING  6


#define PIECENAMES " PRNBQK"

#define WHITE 0
#define BLACK 1
#define EMPTY 2

#define NEGATE(c) ((c)^1)

#define CHECKMATE 3
#define STALEMATE 4
#define KINGOK 5
#define CHECK 1
#define PROMO 1 << 1
#define CAPTURE 1 << 2
#define NONCAPTURE 1 << 3


/* Macros for puttin together color and rank, returning color or rank
   and finding the same for a certain position on a board. and for
   getting the piece at a certain position */

#define MAKE(C,R) ((R<<2) | (C)) 
#define COLOR(X) ((X) & 3)
#define RANK(X) ((X) >> 2)
#define RANKAT(B,X,Y) ((B)[((Y)<<3)+(X)] >> 2)
#define COLORAT(B,X,Y) ((B)[((Y)<<3)+(X)] & 3)
#define PIECEAT(B,X,Y) ((B)[((Y)<<3)+(X)])


#define TILECOLOR(B,X,Y) (((Y)^(X))&1)
#define BSIZE 8
#define noisy false

#define ptoc(p) (PIECENAMES[RANK(p)] | ((COLOR(p) & 1)<<5))
#define abs(x) ((x > 0) ? x : -x)

struct move { 
  int srcx; 
  int srcy; 
  int destx; 
  int desty;
  char promo;
}

struct boardt {
  unsigned byte b[BSIZE*BSIZE];
  unsigned byte flags;
  byte wenpassant;
  byte benpassant;
  int nextturn;
  int blackmb;
  int whitemb;
  bool wcastled;
  bool bcastled;
}

/* init(b) initializes the board b to the starting configuraton a chess game */

extern void init(boardt board);

/* validate move(b,m) returns true/false based on whether m is a valid move on board b*/

extern bool validate_move(boardt board, move m); 

/* execute_move(b,m) performs the move m on the board b. Note that it does not check
   the validity of the move m.*/

extern void execute_move(boardt board, move m);

/* gameover(b) returns either WHITE, BLACK, STALEMATE or KINGOK based on whether 
   white has won, black has won, the board is a stalemate or the game is not over,
   respectively. */

extern int gameover(boardt board);

/* check(b) returns true iff the king of the current player is in check */

extern bool check(boardt board);

extern void printboard(boardt board, int indent);

/* checky(b,m) returns true if the move m is a valid move for the piece at (m.src,m.dest).
   But it does not check if this move is valid in terms of being in check, moving in/out of check,
   castling, etc. */
extern bool checky(boardt board, move m);

/* copyboard(b) returns a copy of b */
extern boardt copyboard(boardt board);

/* initialize a board using standard FEN notation 
   returns nonzero on success
*/
extern int init_fen(boardt board, string fen);


/* convert the move to a printable string */
extern string mtos(move m);

/*convert a string to a move */
extern move stom(string);


/* Convert the board to a fen string */
extern string board_to_fen(boardt board);


#endif
